#! /bin/bash

INDEX_NAME=asset
ALIAS_NAME=a_asset
SETTINGS_FILE_NAME=_settings.json
MAPPINGS_FILE_NAME=_mappings.json
ALIASES_FILE_NAME=_aliases.json

function red () {
  printf "\033[1;31m"
}

function green {
  printf "\033[1;32m"
}

function yellow {
  printf "\033[1;33m"
}

function reset () {
  printf "\033[0m"
}

function print_tab {
    printf "\t"
}

function print_newline {
    echo
}

function read_settings {
    print_newline
    SETTINGS_FILE="../settings/${INDEX_NAME}${SETTINGS_FILE_NAME}"
    SETTINGS=''

    if [ -e  "${SETTINGS_FILE}" ]
    then
        print_tab
        echo "Using settings file: ${SETTINGS_FILE}"
        load_settings_json
    else
        printf "\t Settings file not found %s \n" "${SETTINGS_FILE}"
    fi
}

function load_settings_json() {
    print_tab
    SETTINGS=$(sed -e "s/\${NUM_SHARDS}/${NUM_SHARDS}/g" -e "s/\${NUM_REPLICAS}/${NUM_REPLICAS}/g" -e "s/\${SHARDS_PER_NODE}/${SHARDS_PER_NODE}/g" "${SETTINGS_FILE}")
}

function read_mappings {
    MAPPINGS_FILE="../mappings/${INDEX_NAME}${MAPPINGS_FILE_NAME}"
    MAPPINGS=''

    if [ -e  "${MAPPINGS_FILE}" ]
    then
        print_tab
        echo "Using mappings file: ${MAPPINGS_FILE}"
        MAPPINGS=$(cat "${MAPPINGS_FILE}")
    else
        printf "\t Mapping file not found %s \n" "${MAPPINGS_FILE}"
    fi
}

function read_aliases {
    ALIASES_FILE="../mappings/${INDEX_NAME}${ALIASES_FILE_NAME}"
    ALIASES=''

    if [ -e  "${ALIASES_FILE}" ]
    then
        print_tab
        echo "Using alias file: ${ALIASES_FILE}"
        ALIASES=$(cat "${ALIASES_FILE}")
    else
        printf "\t Alias file not found %s \n" "${ALIASES_FILE}"
    fi
}

function drop_index {
    print_tab
#    yellow
#    read -p "Please confirm to drop the index '${INDEX_NAME}' (y|n)  : " -r confirm_drop_i
#    reset
    confirm_drop_i='N'

    if [[ $confirm_drop_i == 'Y' || $confirm_drop_i == 'y' ]]
    then
        red
        printf "\t **** Dropping %s index ****\n" "${INDEX_NAME}"
        reset
        curl -XDELETE "${PROTOCOL}"://"${HOST}":"${PORT}"/"${INDEX_NAME}"
    else
        printf "\t Index Drop cancelled"
    fi
}

function drop_alias {
    print_newline
    print_tab
#    yellow
#    read -p "Please confirm to drop the alias ${INDEX_NAME} (y|n)  : " -r confirm_drop_a
#    reset
    confirm_drop_a='N'

    if [[ $confirm_drop_a == 'Y' || $confirm_drop_a == 'y' ]]
    then
        red
        printf "\t **** Dropping 'a_%s' alias ****\n" "${INDEX_NAME}"
        reset
        curl -XDELETE "${PROTOCOL}"://"${HOST}":"${PORT}"/"${INDEX_NAME}"/_alias/a_"${INDEX_NAME}"
    else
        printf "\tAlias Drop cancelled"
    fi

    print_newline
}


function create_index {
    print_newline
    green
    printf "\t **** Creating index '%s' at ${HOST}:${PORT} ****\n" "${INDEX_NAME}"
    reset
    create_index_cmd="curl -X PUT -u ${USER} ${PROTOCOL}://${HOST}:${PORT}/${INDEX_NAME} -H 'Content-Type: application/json'"


    if [ -n "${SETTINGS}" ] || [ -n "${MAPPINGS}" ]
    then
        create_index_cmd+=" -d '{ "
    fi

    if [ -n "${SETTINGS}" ]
    then
        create_index_cmd+="\"settings\" :
                ${SETTINGS}"
    fi

    if [ -n "${SETTINGS}" ] && [ -n "${MAPPINGS}" ]
    then
        create_index_cmd+=","
    fi

    if [ -n "${MAPPINGS}" ]
    then
        create_index_cmd+="\"mappings\" :
                ${MAPPINGS}"
    fi

    if [ -n "${SETTINGS}" ] || [ -n "${MAPPINGS}" ]
    then
        create_index_cmd+=" }'"
    fi

    print_tab
#    yellow
#    read -p "Please confirm to create index '${INDEX_NAME}' (y|n)  : " -r confirm_create
#    reset
    confirm_create='Y'

    if [[ $confirm_create == 'Y' || $confirm_create == 'y' ]]
    then
        green
        printf "\t**** Executing create index curl for %s index ****\n" "${INDEX_NAME}"
        reset
        eval "$create_index_cmd"
    else
        printf "\tIndex Creation cancelled"
#        Stopping the execution, dont need to create alias if index creation is cancelled
        exit 0
    fi
}


function create_alias {
    ALIAS_NAME="a_${INDEX_NAME}"
    print_newline
    green
    printf "\t**** Setting up the alias %s for index %s ****\n" "${ALIAS_NAME}" "${INDEX_NAME}"
    reset
    create_alias_cmd="curl -X POST -u ${USER} ${PROTOCOL}://${HOST}:${PORT}/_aliases -H 'Content-Type: application/json' -d '
    {
        \"actions\" : [
            { \"add\" : { \"index\" : \"${INDEX_NAME}\", \"alias\" : \"${ALIAS_NAME}\" } }
        ]
    }'"

#    echo "$create_alias_cmd" - comment command output, it contains password also
    print_tab
#    yellow
#    read -p "Please confirm to create alias '${INDEX_NAME}' (y|n)  : " -r confirm_create
#    reset
    confirm_create='Y'

    if [[ $confirm_create == 'Y' || $confirm_create == 'y' ]]
    then
      print_tab
      eval "$create_alias_cmd"
      print_newline
    else
        printf "\tAlias Creation cancelled"
        exit 0
    fi
}

function add_field_aliases {
    print_newline
    green
    printf "\t **** Index '%s' exist, adding field aliases ****\n" "${INDEX_NAME}"
    add_aliases_cmd="curl -X PUT -u ${USER} ${PROTOCOL}://${HOST}:${PORT}/${INDEX_NAME}/_mapping -H 'Content-Type: application/json'"

    if [ -n "${ALIASES}" ]
    then
      add_aliases_cmd+=" -d '${ALIASES}'"
    fi

    printf "\t**** Executing add field aliases operation for %s index ****\n" "${INDEX_NAME}"
    reset
    eval "$add_aliases_cmd"
}

function is_alias_exist(){
   curl -I --silent -u ${USER} "${PROTOCOL}://${HOST}:${PORT}/a_${INDEX_NAME}" | head -n 1 | awk -F' ' '{print $2}'
}

function is_index_exist(){
   curl -I --silent -u ${USER} "${PROTOCOL}://${HOST}:${PORT}/${INDEX_NAME}" | head -n 1 | awk -F' ' '{print $2}'
}

if [ $# -lt 1 ]
then
    green
    printf "Usage: setup_asset_index.sh -u <user:password> -h <host> -p [port] -i <index_name> -d <drop_existing_index> -a <create_alias>\n -s <num_of_shards> -r <num_of_replicas> -n <shards_per_node>"
    reset
    printf "\t host: ES node_ips , default is localhost \n"
    printf "\t port: ES port , default is 9200 \n"
    printf "\t index name: default is asset \n"
    printf "\t drop_existing_index: true|false , default is false \n"
    printf "\t create_alias_flag: true|false , default is false \n"
    printf "\t num_of_shards: default is 5 \n"
    printf "\t num_of_replicas: default is 1 \n"
    printf "\t shards_per_node: default is 3 \n"
    exit 1
fi

ENV="${ENV:-local}"
HOST=localhost
PORT=9200
DROP_INDEX=false
CREATE_ALIAS=false
CREATE_INDEX=false
PROTOCOL=http

while getopts u:h:p:i:d:a:s:r:n: flag
do
    case "${flag}" in
        u) USER=${OPTARG};;
        h) HOST=${OPTARG};;
        p) PORT=${OPTARG};;
        i) INDEX_NAME=${OPTARG};;
        d) DROP_INDEX=${OPTARG};;
        a) CREATE_ALIAS=${OPTARG};;
        s) NUM_SHARDS=${OPTARG};;
        r) NUM_REPLICAS=${OPTARG};;
        n) SHARDS_PER_NODE=${OPTARG};;
        *) echo "Invalid Flag";
    esac
done

# Workaround for -> -s or -r flags was passed without value or non-numeric value
NUMERIC_RE='^[0-9]+$'
red
if ! [[ $NUM_SHARDS =~ $NUMERIC_RE ]];
then
      printf "\n\t **** Got empty NUM_SHARDS value, setting it to default 6 **** \n"
      NUM_SHARDS=6
fi

if ! [[ $NUM_REPLICAS =~ $NUMERIC_RE ]];
then
      printf "\n\t **** Got empty NUM_REPLICAS value, setting it to default 1 **** \n"
      NUM_REPLICAS=1
fi
if ! [[ $SHARDS_PER_NODE =~ $NUMERIC_RE ]];
then
      printf "\n\t **** Got empty SHARDS_PER_NODE value, setting it to default 3 **** \n"
      SHARDS_PER_NODE=3
fi
reset

export USER HOST PORT DROP_INDEX CREATE_ALIAS CREATE_INDEX INDEX_NAME NUM_SHARDS NUM_REPLICAS SHARDS_PER_NODE
# echo "${USER}" - Don't print it contains password
print_newline
echo "ENV" "HOST" "PORT" "DROP_INDEX" "CREATE_ALIAS" "CREATE_INDEX" "INDEX_NAME" "NUM_SHARDS" "NUM_REPLICAS" "SHARDS_PER_NODE"
print_newline
echo "${ENV}" "${HOST}" "${PORT}" "${DROP_INDEX}" "${CREATE_ALIAS}" "${CREATE_INDEX}" ${INDEX_NAME} "${NUM_SHARDS}" "${NUM_REPLICAS}" "${SHARDS_PER_NODE}"

print_newline

# Added check for env is local.... Index Drop should not be attempted in non-local envs
if [[ $ENV == "local" && $DROP_INDEX == 'true' ]];
then
    drop_alias
    drop_index
fi

if [[ $PORT == "443" ]];
then
    PROTOCOL=https
fi

read_settings
echo "$SETTINGS"
read_mappings
echo "$MAPPINGS"
read_aliases
echo "$ALIASES"

echo "Checking if alias exists"
http_status_code=$(is_alias_exist)
echo $http_status_code
if [[ $http_status_code == 200 ]]
then
  CREATE_ALIAS=false
  green
  printf "\t**** Alias %s already exists, no need to create the alias ****\n" "a_${INDEX_NAME}"
  reset
else
   printf "\t**** Alias %s doesn't exist, going to create alias ****\n" "a_${INDEX_NAME}"
   CREATE_ALIAS=true
fi

echo "Checking if index exists"
http_status_code=$(is_index_exist)
echo $http_status_code
if [[ $http_status_code == 200 ]]
then
  CREATE_INDEX=false
  green
  printf "\t**** Index %s already exists, no need to create the index ****\n" "${INDEX_NAME}"
  reset
else
   printf "\t**** Index %s doesn't exist, going to create index ****\n" "${INDEX_NAME}"
   CREATE_INDEX=true
fi

if $CREATE_INDEX && [[ -n "${SETTINGS}" ]] && [[ -n "${MAPPINGS}" ]] ;
then
    create_index
    if [[ -n "${ALIASES}" ]];
    then
        add_field_aliases
    fi
fi

# sleep for 10 second
sleep 10

if $CREATE_ALIAS ;
then
    create_alias
fi

yellow
printf "\t**** ES Schema Initialization End *****"
reset